package com.yuexiu.secp.read.controller;

import com.alibaba.fastjson.JSONObject;

import com.sie.iot.base.bean.RedisMessageContentBean;
import com.sie.iot.common.bean.PaginationRequestData;
import com.sie.iot.common.bean.ResponseData;
import com.sie.iot.common.iotenum.ResponseMsgCode;
import com.sie.iot.common.model.inter.IBaseCommon;
import com.sie.iot.common.secure.annotation.CheckToken;
import com.sie.iot.common.services.CommonAbstractService;
import com.sie.iot.component.exception.GushenRuntimeException;
import com.sie.iot.distribute.transation.annotation.DistrTransMessageParam;
import com.siefw.hibernate.core.paging.Pagination;
import com.yuexiu.secp.read.dto.AdvertDto;
import com.yuexiu.secp.read.model.entities.AdvertEntity;
import com.yuexiu.secp.read.model.entities.readonly.AdvertEntityRo;
import com.yuexiu.secp.read.model.service.IAdvertServer;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

@Validated
@Api(value = "广告", tags = "广告-维护")
@RestController
@Slf4j
@RequestMapping("/advertController")
public class AdvertController extends CommonAbstractService {
    private static final Logger LOGGER = LoggerFactory.getLogger(AdvertController.class);

     @Autowired
     private IAdvertServer iAdvertServer;

     @Override
     public IBaseCommon<?> getBaseCommonServer() {
        return this.iAdvertServer;
    }

    @ApiOperation("获取广告维护表分页列表")
    @PostMapping(value = "/findAdvertInfoPagination")
    @CheckToken
    public ResponseData<Pagination<AdvertEntityRo>> findAdvertInfoPagination(@RequestBody PaginationRequestData<AdvertDto> paginationRequestData) {
        Integer pageIndex = paginationRequestData.getPageIndex();
        Integer pageRows = paginationRequestData.getPageRows();
        AdvertDto advertDto = paginationRequestData.getParams();
        JSONObject jsonObject = JSONObject.parseObject(JSONObject.toJSONString(advertDto));
        try {
            log.info("分页查询广告列表  参数：query:{}",advertDto);
            Pagination<AdvertEntityRo> pagination = iAdvertServer.findAdvertPagination(jsonObject, pageIndex, pageRows, paginationRequestData.getOrderByBean());
            return ResponseData.data(pagination);
        } catch (Exception e) {
            LOGGER.error(" find - AdvertInfoPagination error:"+e);
            throw new GushenRuntimeException(ResponseMsgCode.ERROR.msgCode, e);
        }
    }

    @ApiOperation(value = "保存或者更新广告列表")
    @PostMapping(value = "/saveOrUpdateAdvertInfo")
    @CheckToken
    public ResponseData saveOrUpdateAdvertInfo(@RequestBody AdvertEntity advertEntity) {
        log.info("保存 参数:{}",advertEntity);
        iAdvertServer.saveOrUpdateAdvertInfo(advertEntity);
        return ResponseData.success();
    }
    @ApiOperation(value = "根据广告表 id 查询详细")
    @GetMapping(value = "/findById")
    @CheckToken
    public ResponseData findById(@RequestParam Long id){
        try {
            log.info("根据 primaryKey 查询广告详细 id：{}", id);
            AdvertEntity entity = iAdvertServer.findById(id);
            return ResponseData.data(entity);
        }catch (Exception e){
            throw new GushenRuntimeException(ResponseMsgCode.ERROR.msgCode, e);
        }
    }
    @ApiOperation(value = "根据广告表 id 删除")
    @GetMapping(value = "/delete")
    @CheckToken
    public ResponseData deleteById(@RequestParam Long id){
        log.info("根据 id 删除 参数：{}",id);
        AdvertEntity advertEntity = iAdvertServer.findById(id);
        if (advertEntity != null) {
            advertEntity.setDeleteFlag(1);
        }
        iAdvertServer.update(advertEntity);
        return ResponseData.success();
    }

    @ApiOperation(value = "广告申请流程使用")
    @PostMapping(value = "/doCommit")
    @CheckToken
    public ResponseData doCommit(@RequestBody AdvertEntity advertEntity,@RequestParam String processDefinitionKey) {
        log.info("广告维护申请流程 参数:{}", advertEntity);
        iAdvertServer.doCommit(advertEntity,processDefinitionKey);
        return ResponseData.success();
    }
    @ApiOperation(value = "广告流程回调使用")
    @PostMapping(value = "/updateAdvertState")
    @CheckToken
    public ResponseData updateAdvertState(@DistrTransMessageParam Long msgId, RedisMessageContentBean redisMessageContentBean) {
        log.info("广告维护申请流程回调 参数:{}", redisMessageContentBean);
        iAdvertServer.updateAdvertState(msgId,redisMessageContentBean);
        return ResponseData.success();
    }
}
