package com.yuexiu.secp.read.model.service.impl;


import com.alibaba.fastjson.JSONObject;
import com.sie.iot.base.bean.RedisMessageContentBean;
import com.sie.iot.common.bean.OrderByBean;
import com.sie.iot.common.model.inter.server.BaseCommonServer;
import com.sie.iot.common.util.SaafToolUtils;

import com.sie.iot.distribute.transation.annotation.DistrTransMessageConsumer;
import com.sie.iot.distribute.transation.annotation.DistrTransMessageParam;
import com.sie.iot.distribute.transation.annotation.DistrTransMessageProvider;
import com.siefw.hibernate.core.dao.ViewObject;
import com.siefw.hibernate.core.paging.Pagination;
import com.yuexiu.secp.read.infra.config.BpmProcessingUtil;
import com.yuexiu.secp.read.infra.config.WorkflowApprovalStatus;
import com.yuexiu.secp.read.model.dao.readonly.AdvertDaoRO;
import com.yuexiu.secp.read.model.entities.AdvertEntity;
import com.yuexiu.secp.read.model.entities.FlowProcessCallbackEntity;
import com.yuexiu.secp.read.model.entities.readonly.AdvertEntityRo;
import com.yuexiu.secp.read.model.service.IAdvertServer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import java.util.HashMap;
import java.util.Map;

@Component("advertServer")
public class AdvertServer extends BaseCommonServer<AdvertEntity> implements IAdvertServer {

    @Autowired
    private ViewObject<AdvertEntity> advertDao;
    @Autowired
    private AdvertDaoRO advertDaoRO;
    @Autowired
    private BpmProcessingUtil bpmProcessingUtil;

    /***********流程标识***********/
    /**
     * 请假
     */
    public final static String QJSQD = "OABUS.QINGJ";

    @Override
    public Pagination<AdvertEntityRo> findAdvertPagination(JSONObject jsonObject, Integer pageIndex, Integer pageRows, OrderByBean orderByBean) {
        StringBuffer sql = new StringBuffer(AdvertEntityRo.QUERY);
        Map<String,Object> map = new HashMap<String,Object>();
        //拼接查询参数
        SaafToolUtils.parperHbmParam(AdvertEntityRo.class,jsonObject,sql,map);
        SaafToolUtils.changeQuerySort(jsonObject, sql, "ya.LAST_UPDATE_DATE desc", false); //排序
        Pagination<AdvertEntityRo> findListResult = advertDaoRO.findPagination(sql,map,pageIndex,pageRows);
        return findListResult;
    }
    @Override
    public AdvertEntity findById(Long id) {
        AdvertEntity advertEntity = advertDao.getById(id);
        return advertEntity;
    }

    @Override
    public AdvertEntity saveOrUpdateAdvertInfo(AdvertEntity advertEntity) {
        try {
            //判断更新还是保存
            if(advertEntity.getAdvertId() != null ){
                advertDao.update(advertEntity);
            }else {
                advertDao.save(advertEntity);
            }
        } catch (Exception e){
            e.printStackTrace();
        }
          return  advertEntity;
    }
    @Override
    @DistrTransMessageProvider(desc = "leaveProvider")
    public void doCommit(AdvertEntity advertEntity,String processDefinitionKey) {
        //提交之前先进行保存
        this.saveOrUpdateAdvertInfo(advertEntity);

        //封装流程参数
        Map<String,Object> map = new HashMap<>();
        map.put("processDefinitionKey",processDefinitionKey);//必传 流程定义Key 指定启动哪个流程
        map.put("saveonly",false);//saveonly : 为true提交流程，流程不会走到下一个节点，为false流程会走到下一个节点
        map.put("businessKey", advertEntity.getAdvertId());//必传 传入业务主键
        map.put("title","广告维护申请流程："+ advertEntity.getAdvertId());
        map.put("orgId", advertEntity.getTenantId());//旧版传 记录租户id
        map.put("companyId", advertEntity.getTenantId());//必传 记录租户id
        //map.put("systemType","demo");//非必要
        map.put("applyPersonId", advertEntity.getCreatedBy());//必传 申请人id

//        //非必传 ，这种是表单参数  流程变量中有用到表单字段需要在此传入
//        List<Map<String,Object>> variablesList = new ArrayList<>();
//        Map<String,Object> startUserId = new HashMap<>();
//        startUserId.put("name","id");
//        startUserId.put("type","Long");
//        startUserId.put("value", advertEntity.getAdvertId());
//        map.put("variables",variablesList);// 非必传 ，流程变量中用到的变量需要在此传入

        if ("草稿".equals(advertEntity.getAdvertState())){
            //提交流程
            bpmProcessingUtil.asynchronousToBpm(map);
        }
    }
    /**
     * 工作流每走一步都会在这里有一条消息  判断不同的状态  更新不同的业务表
     * @param msgId
     * @param redisMessageContentBean
     */
    @Override
    @DistrTransMessageConsumer(desc = "updateAdvertState")
    public void updateAdvertState(@DistrTransMessageParam Long msgId, RedisMessageContentBean redisMessageContentBean) {
        String payload = redisMessageContentBean.getMessageBody();
        FlowProcessCallbackEntity callbackEntity = JSONObject.parseObject(payload, FlowProcessCallbackEntity.class);
        //看下是不是 请假的流程标识
        if (QJSQD.equals(callbackEntity.getProcDefKey())){
            AdvertEntity entity = advertDao.getById(callbackEntity.getId());
            //审批通过
            if (WorkflowApprovalStatus.ALLOW.getIndex().equals(callbackEntity.getStatus())){
                entity.setAdvertState(001);
                //处理其他业务  发邮件 什么的
                advertDao.update(entity);
                //驳回
            } else if (WorkflowApprovalStatus.REFUSAL.getIndex().equals(callbackEntity.getStatus())){
                entity.setAdvertState(002);
                advertDao.update(entity);
                //审批中
            }else if (WorkflowApprovalStatus.APPROVAL.getIndex().equals(callbackEntity.getStatus())){
                entity.setAdvertState(003);
                advertDao.update(entity);
            }
        }
    }
}
